/**
 * Copyright (C) 2020 Wasabeef
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * See about document.execCommand: https://developer.mozilla.org/en-US/docs/Web/API/Document/execCommand
 */

var RE = {};

RE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0};

RE.editor = document.getElementById('editor');

RE.editor.addEventListener("keydown", function(event) {
  var key = event.key || event.keyCode;
  if (key === "Delete" || key === "Backspace") {
          var selection = window.getSelection();
          var anchorNode = selection.anchorNode;
          if (anchorNode&&anchorNode.nodeType == Node.TEXT_NODE) {
              const range = selection.getRangeAt(0);
              const startContainer = range.startContainer;
              const startOffset = range.startOffset;
              var textContent = anchorNode.textContent;
              //\u00A0
              // 判断文本内容是否包含零宽字符
              if (textContent === '\uFEFF') {
//              if (textContent === '\u00A0') {
                  console.log('焦点位于零宽字符上，拦截了 keydown 事件');
                  const previousNode = anchorNode.previousElementSibling;
                  if (previousNode && previousNode.nodeName === 'SPAN' && previousNode.classList.contains('k-contact-at')) {
                      event.preventDefault();
                      console.log('光标在 "k-contact-at" 的 <span> 标签之后');
                      const parentNode = previousNode.parentNode;
                      parentNode.removeChild(previousNode);
                      const anchorNodeParent = anchorNode.parentNode;
                      anchorNodeParent.removeChild(anchorNode);
                      console.log('已删除@其他人节点');
                  }
              }
          }
  }
});

document.addEventListener("selectionchange", function() {
    try{
        var a = restoreStyle();
        RE.backuprange();
        if(document.getSelection()==null||document.getSelection()==""){
            followState(a);
        }
        RE.enabledEditingItems();
    }catch(error){
        console.log("lsy"+error)
    }
 });

// Initializations
RE.callback = function() {
    window.location.href = "re-change://" + encodeURIComponent(RE.getHtml());
    var html=RE.getHtml();
    if(html==""||html=="<p><br></p>"){
        RE.showBackTxt();
    }else{
        RE.clearBackTxt();
    }
    txtChangeCallBack();
}

var txtChangeCallBack = debounce(txtChange, 400);

//function txtChange(){
//    console.log("txtchange");
//    window.location.href = "re-change://" + encodeURIComponent(RE.getHtml());
//}


function txtChange(){
    var s="";
//    var p = document.getElementsByTagName("p");
//    for(var i = 0;i < p.length ; i++){
//        var a = p[i].innerText;
//        s=s+a;
//    }
//    var blockquote = document.getElementsByTagName("blockquote");
//    for(var i = 0;i < blockquote.length ; i++){
//        var a = blockquote[i].innerText;
//        s=s+a;
//    }

    var precedingText="";
    // 获取当前选中的文本范围
    var selection = window.getSelection();
    // 获取光标位置
    var anchorNode = selection.anchorNode;
    var anchorOffset = selection.anchorOffset;
    // 确保光标位置在文本节点内
    if (anchorNode&&anchorNode.nodeType === Node.TEXT_NODE) {
      var textContent = anchorNode.textContent;
      // 计算截取的起始位置
      var startIndex = Math.max(0, anchorOffset - 11);
      // 提取光标前的指定长度的文字内容
      precedingText = textContent.substring(startIndex, anchorOffset);
    }else{
        precedingText="";
    }

    var s_emoji = document.getElementsByClassName("w-e-emoji-eleImg-small").length;
    var b_emoji = document.getElementsByClassName("w-e-emoji-eleImg-big").length;
    var img = document.getElementsByClassName("w-e_img-card").length;
    var card_link = document.getElementsByClassName("w-e_link-card").length;
    var line = document.getElementsByClassName("w-e_line-card").length;

    var emojiNum = b_emoji+s_emoji;
    var cardNum = img+card_link+line
//    console.log("~~~~"+s+":+"+emojiNum+":+"+cardNum);
    window.heroweb.txtContent(RE.getHtml(),precedingText,emojiNum,cardNum,img);

}

function debounce(fn, delay) {
    var timer; // 维护一个 timer
    return function () {
        var _this = this; // 取debounce执行作用域的this
        if (timer) {
            clearTimeout(timer);
        }
        timer = setTimeout(function () {
            fn.apply(_this); // 用apply指向调用debounce的对象，相当于_this.fn(args);
        }, delay);
    };
}

RE.handleNum = function() {
    txtChange();
}

RE.setHtml = function(contents) {
    RE.editor.innerHTML = decodeURIComponent(contents.replace(/\+/g, '%20'));
}

RE.getHtml = function() {
    return RE.editor.innerHTML;
}
RE.clearText = function(txt) {
    var selection = window.getSelection();
    var length=txt.length;
    var anchorNode = selection.anchorNode;
    var anchorOffset = selection.anchorOffset;
    // 确保光标位置在文本节点内
    if (anchorNode.nodeType === Node.TEXT_NODE) {
      var textContent = anchorNode.textContent;
      // 计算删除的起始位置
      var startIndex = Math.max(0, anchorOffset - length);
      // 计算删除的字符数
      var deleteCount = anchorOffset - startIndex;
            // 计算截取的起始位置
      var startIndex = Math.max(0, anchorOffset - length);
      // 提取光标前的指定长度的文字内容
      precedingText = textContent.substring(startIndex, anchorOffset);
      if(precedingText==txt){
            // 删除光标前面的文字
            var newTextContent = textContent.substring(0, startIndex) + textContent.substring(anchorOffset);

            // 更新文本节点的内容
            anchorNode.textContent = newTextContent;

            // 调整光标位置
            var newOffset = anchorOffset - deleteCount;
            selection.collapse(anchorNode, newOffset);
      }
//    console.log("lsy~"+precedingText+":+:"+txt+"   "+(precedingText==txt));
    }
}

RE.insertAt = function(name,id) {
    // 获取当前光标位置的 Selection 对象
    var selection = window.getSelection();
    if(selection.rangeCount>0){
        var range = selection.getRangeAt(0);
        // 创建包含不可选中的 "@" 符号和相关文字的元素
        var container = document.createElement('span');
        container.contentEditable = 'false';
        container.style.userSelect = 'none';
        container.style.cursor = 'pointer';
        container.style.webkitUserSelect = 'none';
        container.style.MozUserSelect = 'none';
        container.style.msUserSelect = 'none';

        container.setAttribute("data-k-user-id", id);

        container.classList.add("k-contact-at");


        container.style.color = '#1DB5B8';

        container.addEventListener('click', function() {
          // 在点击事件中执行你想要的操作
            window.heroweb.clickAt(name);
            console.log('点击了不可选中的 @ 符号');
        });

//        var atSymbol = document.createElement('span');
//        atSymbol.innerHTML = '&#64;'; // 使用 HTML 实体编码表示 "@"
//        container.appendChild(atSymbol);


        var spaceText1 = '\u00A0';
        var textNode1 = document.createTextNode(spaceText1);
        range.insertNode(textNode1);

        var spaceText = '\uFEFF';
        var textNode = document.createTextNode(spaceText);
        range.insertNode(textNode);



        var text = document.createTextNode('@'+name);
        container.appendChild(text);
        // 在光标位置插入元素
        range.insertNode(container);


        // 创建一个新的范围对象并将其定位到新插入的文本节点
        var newRange = document.createRange();
        newRange.setStartAfter(textNode1);
        newRange.collapse(true);

       // 创建一个新的选择对象并将范围设置为新插入的文本节点
       var selection = window.getSelection();
       selection.removeAllRanges();
       selection.addRange(newRange);
       RE.handleNum();
    }
}

RE.getText = function() {
    return RE.editor.innerText;
}

RE.setBaseTextColor = function(color) {
    RE.editor.style.color  = color;
}

RE.setBaseFontSize = function(size) {
    RE.editor.style.fontSize = size;
}

RE.setPadding = function(left, top, right, bottom) {
  RE.editor.style.paddingLeft = left;
  RE.editor.style.paddingTop = top;
  RE.editor.style.paddingRight = right;
  RE.editor.style.paddingBottom = bottom;
}

RE.setBackgroundColor = function(color) {
    document.body.style.backgroundColor = color;
}

RE.setBackgroundImage = function(image) {
    RE.editor.style.backgroundImage = image;
}

RE.setWidth = function(size) {
    RE.editor.style.minWidth = size;
}

RE.setHeight = function(size) {
    RE.editor.style.height = size;
}

RE.setTextAlign = function(align) {
    RE.editor.style.textAlign = align;
}

RE.setVerticalAlign = function(align) {
    RE.editor.style.verticalAlign = align;
}

RE.setPlaceholder = function(placeholder) {
    RE.editor.setAttribute("placeholder", placeholder);
}

RE.setInputEnabled = function(inputEnabled) {
    RE.editor.contentEditable = String(inputEnabled);
}

RE.undo = function() {
    document.execCommand('undo', false, null);
    RE.enabledEditingItems()
}

RE.redo = function() {
    document.execCommand('redo', false, null);
    RE.enabledEditingItems()
}

RE.insertText= function(text){
        if (document.queryCommandSupported('insertText')) {
            document.execCommand('insertText', false, text);
        } else {
            document.execCommand('paste', false, text);
        }
}

var selectedIsBold=false;
RE.setBold = function(needBold) {
    // 检查当前文本是否为粗体
    var isBold = document.queryCommandState('bold');
    if(isBold!=needBold){
      document.execCommand('bold', false, null);
    }
    selectedIsBold=needBold;
    RE.enabledEditingItems()
}

var selectedIsItalic=false;
RE.setItalic = function(needItalic) {
    var isItalic = document.queryCommandState('italic');
    if(needItalic!=isItalic){
        document.execCommand('italic', false, null);
    }
    selectedIsItalic=needItalic;
    RE.enabledEditingItems()
}

var selectedIsUnderline=false;
RE.setUnderline = function(needUnderline) {
    var isUnderline = document.queryCommandState('underline');
    if(isUnderline!=needUnderline){
        document.execCommand('underline', false, null);
    }
    selectedIsUnderline=needUnderline;
     RE.enabledEditingItems()
}

RE.setTextColor = function(color) {
//    RE.restorerange();
    selectedColor=color;
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('foreColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setFontSize = function(fontSize){
    selectedFontSize=fontSize;
    document.execCommand("fontSize", false, fontSize);
}

RE.setJustifyLeft = function() {
    setAtEditable("true");
    document.execCommand('justifyLeft', false, null);
    setAtEditable("false");
}

RE.setJustifyCenter = function() {
    setAtEditable("true");
    document.execCommand('justifyCenter', false, null);
    setAtEditable("false");
}

RE.setJustifyRight = function() {
    setAtEditable("true");
    document.execCommand('justifyRight', false, null);
    setAtEditable("false");
}

function setAtEditable(editable){
    var elements=document.getElementsByClassName("k-contact-at");
    for(var i = 0;i < elements.length ; i++){
         elements[i].setAttribute("contenteditable",editable);
    }
}

RE.setBlockquote = function() {
    var a = getHtml(window);
    if(a.length>0&&a.indexOf('div')!=-1){
       window.heroweb.selectBlock(a);
    }else{
       document.execCommand('formatBlock', false, '<blockquote>');
       RE.enabledEditingItems();
    }
}

function getHtml(win){
    if (win.getSelection) {
        var range=win.getSelection().getRangeAt(0);
        var container = win.document.createElement('div');

        container.appendChild(range.cloneContents());

        return container.innerHTML;

    }else if (win.document.getSelection) {
        var range=win.getSelection().getRangeAt(0);

        var container = win.document.createElement('div');

        container.appendChild(range.cloneContents());

        return container.innerHTML;

    }else if (win.document.selection) {
        return win.document.selection.createRange().htmlText;

    }

}

RE.insertImage = function(url, alt) {
    var html = '<img src="' + url + '" alt="' + alt + '" />';
    RE.insertHTML(html);
}

RE.insertImageW = function(url, alt, width) {
    var html = '<img src="' + url + '" alt="' + alt + '" width="' + width + '"/>';
    RE.insertHTML(html);
}

RE.insertImageWH = function(url, alt, width, height) {
    var html = '<img src="' + url + '" alt="' + alt + '" width="' + width + '" height="' + height +'"/>';
    RE.insertHTML(html);
}

RE.insertVideo = function(url, alt) {
    var html = '<video src="' + url + '" controls></video><br>';
    RE.insertHTML(html);
}

RE.insertVideoW = function(url, width) {
    var html = '<video src="' + url + '" width="' + width + '" controls></video><br>';
    RE.insertHTML(html);
}

RE.insertVideoWH = function(url, width, height) {
    var html = '<video src="' + url + '" width="' + width + '" height="' + height + '" controls></video><br>';
    RE.insertHTML(html);
}

RE.insertAudio = function(url, alt) {
    var html = '<audio src="' + url + '" controls></audio><br>';
    RE.insertHTML(html);
}

RE.insertYoutubeVideo = function(url) {
    var html = '<iframe width="100%" height="100%" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertYoutubeVideoW = function(url, width) {
    var html = '<iframe width="' + width + '" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertYoutubeVideoWH = function(url, width, height) {
    var html = '<iframe width="' + width + '" height="' + height + '" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertHTML = function(html) {
    RE.restorerange();
    document.execCommand('insertHTML', false, html);
    var html=RE.getHtml();
    console.log("**********"+html);
}

RE.insertLink = function(type,des, title,json) {
    var sel = document.getSelection();
    if (sel.toString().length == 0) {
        var str="";
            if(type==1){
                str= '<a href="#" data-postid="'+des+'" onclick="clickPostCard(this)">'+title+'</a>'
            }else if(type==2){
                str= '<a href="#" data-topicid="'+des+'" onclick="clickTopicCard(this)">'+title+'</a>'
            }else{
                str= '<a href="#" data-link="'+des+'" onclick="clickNetCard(this)">'+title+'</a>'
            }
            RE.insertHTML(str);
    } else if (sel.rangeCount) {
       var el = document.createElement("a");
       el.setAttribute("href", "javascript:void(0)");
       el.setAttribute("title", title);
       el.setAttribute("data-des", json);
       if(type==1){
            el.setAttribute("data-postid", des);
            el.setAttribute("ontouchend", "clickPostCard(this)");
       }else if(type==2){
            el.setAttribute("data-topicid", des);
            el.setAttribute("ontouchend", "clickTopicCard(this)");
       }else{
            el.setAttribute("data-link", des);
            el.setAttribute("ontouchend", "clickNetCard(this)");
       }
       var range = sel.getRangeAt(0).cloneRange();
       range.surroundContents(el);
       sel.removeAllRanges();
       sel.addRange(range);
   }
}

RE.exec = function(command) {
    //执行指令
    var _self = this;
    var value = '<' + command + '>';
    document.execCommand('formatBlock', false, value);
}

RE.execnoblock = function() {
    var a = getHtml(window);
    if(a.length>0&&a.indexOf('div')!=-1){
        window.heroweb.selectBlock(a);
    }else{
        var value = '<p>';
        document.execCommand('formatBlock', false, value);
     }
}


RE.setTodo = function(text) {
    var html = '<input type="checkbox" name="'+ text +'" value="'+ text +'"/> &nbsp;';
    document.execCommand('insertHTML', false, html);
}

RE.prepareInsert = function() {
    RE.backuprange();
}

RE.backuprange = function(){
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
      var range = selection.getRangeAt(0);
      RE.currentSelection = {
          "startContainer": range.startContainer,
          "startOffset": range.startOffset,
          "endContainer": range.endContainer,
          "endOffset": range.endOffset};
    }
}

RE.restorerange = function(){
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.setStart(RE.currentSelection.startContainer, RE.currentSelection.startOffset);
    range.setEnd(RE.currentSelection.endContainer, RE.currentSelection.endOffset);
    selection.addRange(range);
}


var selectedColor
 function getTextForeColor (){
    if(selectedColor==null){
        return document.queryCommandValue('foreColor')
    }
    return selectedColor;
}

var selectedFontSize
function getTextFontSize(){
    if(selectedFontSize==null){
        return document.queryCommandValue('fontSize')
    }
    return selectedFontSize;
}

RE.enabledEditingItems = function(e) {
    var items = restoreStyle();
    window.location.href = "re-state://" + encodeURI(items);
}

RE.focus = function() {
    var range = document.createRange();
    range.selectNodeContents(RE.editor);
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    RE.editor.focus();
}

RE.blurFocus = function() {
    RE.editor.blur();
}

RE.removeFormat = function() {
    document.execCommand('removeFormat', false, null);
}

RE.removeTxt = function(){
    var execed=false
              var selection = window.getSelection();
              var anchorNode = selection.anchorNode;
              if (anchorNode&&anchorNode.nodeType == Node.TEXT_NODE) {
                  var textContent = anchorNode.textContent;
                  if (textContent === '\uFEFF') {
                      const previousNode = anchorNode.previousElementSibling;
                      if (previousNode && previousNode.nodeName === 'SPAN' && previousNode.classList.contains('k-contact-at')) {
                          const parentNode = previousNode.parentNode;
                          parentNode.removeChild(previousNode);
                          const anchorNodeParent = anchorNode.parentNode;
                          anchorNodeParent.removeChild(anchorNode);
                          execed=true
                      }
                  }
              }
    if(!execed){
        document.execCommand("delete",false,null)
    }
}


// Event Listeners
RE.editor.addEventListener("input", RE.callback);
RE.editor.addEventListener("keyup", function(e) {
    if (RE.editor.innerHTML.length == 0) {
        RE.editor.innerHTML = "<p><br></p>"
    }
    var KEY_LEFT = 37, KEY_RIGHT = 39;
    if (e.which == KEY_LEFT || e.which == KEY_RIGHT||e.which == 8 ||e.which == 13) {
        RE.enabledEditingItems(e);
        txtChangeCallBack();
    }

//    if (e.which == 13) {
//       document.execCommand('formatBlock', false, '<p>');
//    }
});

RE.editor.addEventListener('focus', function() {
            // 输入时获取html的内容，length = 0添加p标签
            if (RE.editor.innerHTML.length == 0) {
                RE.editor.innerHTML = "<p><br></p>"
            }
}, false);


RE.editor.addEventListener('blur', function() {
//             RE.editor.saveRange();
             if (RE.editor.innerHTML == "<p><br></p>") {
                 RE.editor.innerHTML = ""
             }
}, false);


//RE.editor.addEventListener("click", RE.enabledEditingItems);

//window.onload=function(){
//window.heroweb.loadFinish();
//}
document.addEventListener('DOMContentLoaded',function(){
window.heroweb.loadFinish();
},false)

RE.setBackTxt = function(str){
    var target=document.getElementById("placeholder-text");
    target.textContent =str
}
RE.showBackTxt = function(){
    var target=document.getElementById("placeholder-text");
    target.style.display="block";
}

RE.clearBackTxt = function(){
    var target=document.getElementById("placeholder-text");
    target.style.display="none";
}


function insertContentHtml (str,isScroll){
     var formatBlock = document.queryCommandValue('formatBlock');
     if (formatBlock.length > 0&&formatBlock=="blockquote") {
          RE.insertHTML("&zwj;");
          RE.prepareInsert();
     }
     RE.insertHTML(str);
     RE.autoScroll(isScroll);
}
function followState (a){
//    if(a.indexOf("justifyLeft")!=-1){
//        RE.setJustifyLeft();
//    }
//    if(a.indexOf("justifyCenter")!=-1){
//        RE.setJustifyCenter();
//    }
//    if(a.indexOf("justifyRight")!=-1){
//        RE.setJustifyRight();
//    }
    RE.setBold(selectedIsBold);
    RE.setUnderline(selectedIsUnderline);
    RE.setItalic(selectedIsItalic);
    RE.setTextColor(getTextForeColor());
    RE.setFontSize(getTextFontSize());
}

function restoreStyle (){
    var items = [];
    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }
    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }
    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    return items.join('@');
}

RE.insertImg =function(imgUrl,width,height){
    var imgStr =
     '<p><br></p>'+ '<div class="w-e_img-card" tabIndex="-1" data-module-name="img-card" has-delete="true" contenteditable="false" >'
      +'<img class="w_e_network_image" img-des="'+width+','+height+','+imgUrl+'" listen-img-load="true" src="'+ imgUrl +'" ontouchend="clickNetImg(this)" onerror="WEImageLoadError(this)" onload="WEImageLoadSuccess(this)">'
      +'<img id="img-delete" src="baseImg/post_delete.png" style="visibility: visible;" ontouchend="removeImg(this.parentNode)"/>'
//      +'<img id="img-cover-select" src="baseImg/set_cover.png" style="visibility: visible;" onclick="editCover("'+ imgUrl +'")"/>'
      + '</div>'
      +'<p><br></p>';
      insertContentHtml(imgStr,true);

}

RE.insertImg2 =function(imgUrl,width,height,isGif){
    var imgStr =
     '<p><br></p>'+ '<div class="w-e_img-card" tabIndex="-1" data-module-name="img-card" has-delete="true" contenteditable="false" >'
      +'<img class="w_e_network_image" img-des="'+width+','+height+','+imgUrl+'" listen-img-load="true" src="'+ imgUrl +'" ontouchend="clickNetImg(this)" onerror="WEImageLoadError(this)" onload="WEImageLoadSuccess(this)">'
      +'<img id="img-delete" src="baseImg/post_delete.png" style="visibility: visible;" ontouchend="removeImg(this.parentNode)"/>'
      +'<img id="img-cover-tag" src="baseImg/cover_tag.png" style="visibility: hidden;" />'
//      +'<img id="img-cover-select" src="baseImg/set_cover.png" style="visibility: visible;" onclick="editCover("'+ imgUrl +'")"/>'
      +(isGif=="0"?'<img id="img-cover-select" src="baseImg/set_cover.png" style="visibility: visible;" alt="'+ imgUrl +'" onclick="editCover(this)"/>':'')
      +(isGif=="0"?'<img id="img-cover-cancel" src="baseImg/ic_cancel_cover.png" style="visibility: hidden;" alt="'+ imgUrl +'" onclick="cancelCover(this)"/>':'')
      +(isGif=="0"?'<img id="img-cover-re-edit" src="baseImg/ic_re_edit_cover.png" style="visibility: hidden;" alt="'+ imgUrl +'" onclick="reCover(this)"/>':'')
      + '</div>'
      +'<p><br></p>';
      insertContentHtml(imgStr,true);

}

RE.insertLine =function(imgUrl,width,height){
    var imgStr =
     '<p><br></p>'+'<div class="w-e_line-card" tabIndex="-1" data-module-name="line-card" has-delete="true" contenteditable="false" >'
           +'<img class="w_e_line_image" img-des="'+width+','+height+','+imgUrl+'" width="100%" src="'+ imgUrl +'">'
           +'<img id="img-delete" src="baseImg/post_delete.png" style="visibility: visible;" ontouchend="removeImg(this.parentNode)"/>'
           + '</div>'
          +'<p><br></p>';
    insertContentHtml(imgStr,false);
}

RE.insertEmoji =function(isBig,emojiString,desc){
    var a = restoreStyle();
    var emojiStr='';
    var alt=desc.replace("_","");
    if(isBig){
        emojiStr ='<img class="w-e-emoji-eleImg-big"  alt="alt-'+alt+'" src="img/'+ emojiString + '">';
    }else{
       emojiStr ='<img class="w-e-emoji-eleImg-small"  alt="alt-'+alt+'" src="img/'+ emojiString + '">';
    }

    RE.insertHTML(emojiStr);
    RE.emojiAutoScroll();
    followState(a);
    console.log("****"+a);
}

RE.insertEmoji2 =function(isBig, emojiPath, desc){
    var a = restoreStyle();
    var emojiStr='';
    var alt=desc.replace("_","");
    if(isBig){
        emojiStr ='<img class="w-e-emoji-eleImg-big"  alt="alt-'+alt+'" src="'+ emojiPath + '">';
    }else{
       emojiStr ='<img class="w-e-emoji-eleImg-small"  alt="alt-'+alt+'" src="'+ emojiPath + '">';
    }

    RE.insertHTML(emojiStr);
    RE.emojiAutoScroll();
    followState(a);
    console.log("****"+a);
}
RE.setImgIsCover=function(url){
    var elements = document.querySelectorAll(".w-e_img-card");
    for (var i = 0; i < elements.length; i++) {
      var divElement = elements[i];
       var cover = divElement.querySelector("#img-cover-select");
       if(cover!=null){
           if(cover.alt==url){
                cover.style.visibility = "hidden";
                 var cancel = divElement.querySelector("#img-cover-cancel");
                 cancel.style.visibility = "visible";
                  var re = divElement.querySelector("#img-cover-re-edit");
                   re.style.visibility = "visible";
                    var tag = divElement.querySelector("#img-cover-tag");
                 tag.style.visibility = "visible";

           }else{
                cover.style.visibility = "visible";
                         var cancel = divElement.querySelector("#img-cover-cancel");
                         cancel.style.visibility = "hidden";
                          var re = divElement.querySelector("#img-cover-re-edit");
                           re.style.visibility = "hidden";
                           var tag = divElement.querySelector("#img-cover-tag");
                                            tag.style.visibility = "hidden";
           }
       }
    }
}

RE.insertEmoji3 =function(isBig, emojiPath, desc){
    var a = restoreStyle();
    var emojiStr='';
    console.log("****"+emojiPath);
    var alt=desc.replace("_","");
    if(isBig){
        emojiStr ='<img class="w-e-emoji-eleImg-big"  alt="alt-'+alt+'" src="'+ emojiPath + '">';
    }else{
       emojiStr ='<img class="w-e-emoji-eleImg-small"  alt="alt-'+alt+'" src="'+ emojiPath + '">';
    }

    RE.insertHTML(emojiStr);
    RE.emojiAutoScroll();
    followState(a);
}

RE.insertLinkHasTitle =function(linkUrl,title,imgUrl,isCustomTitle){
    var linkStr =
     '<p><br></p>'+ '<div class="w-e_link-card" card-des="'+linkUrl+',&,'+title+'" data-module-name="link-card" has-delete="true" contenteditable="false" >'
      +'<div class="w-e_link-card_link-wrap" >'
      +'<img class="w-e_link-card_img" src="'+imgUrl+'" >'
      +'<div class="w-e_link-card_detail" >'
      +'<span class="w-e_link-card_title w-e_hide-font--nowrap" custom-title="'+isCustomTitle+'">'+title+'</span>'
      +'<span class="w-e_link-card_address w-e_hide-font--nowrap">'+linkUrl+'</span>'
      + '</div>'
      + '</div>'
      +'<img id="img-delete" src="baseImg/post_delete.png" style="visibility: visible;" ontouchend="removeImg(this.parentNode)"/>'
      + '</div>'
      +'<p><br></p>';
      console.log("@@@@@@@@111"+linkStr);
      insertContentHtml(linkStr,true);
}


RE.insertLinkTopic =function(topicIdDes,title,content,imgUrl){
    var linkStr =
     '<p><br></p>'+ '<div class="w-e_link-card" card-des="'+topicIdDes+'" data-module-name="link-card" has-delete="true" contenteditable="false" >'
      +'<div class="w-e_link-card_link-wrap" >'
      +'<img class="w-e_link-card_img" src="'+imgUrl+'" >'
      +'<div class="w-e_link-card_detail" >'
      +'<span class="w-e_link-card_title w-e_hide-font--nowrap">'+title+'</span>'
      +'<span class="w-e_link-card_address w-e_hide-font--nowrap">'+content+'</span>'
      + '</div>'
      + '</div>'
      +'<img id="img-delete" src="baseImg/post_delete.png" style="visibility: visible;" ontouchend="removeImg(this.parentNode)"/>'
      + '</div>'
      +'<p><br></p>';
      console.log("@@@@@@@@111"+linkStr);
      insertContentHtml(linkStr,true);
}

RE.insertLinkSmallTopic =function(topicIdDes,title,gameId){
       var linkStr =
     '<a class="w-e_link-topic" href="#" data-kr-topicid='+topicIdDes+'  data-kr-gametxt='+gameId+ ' data-des=""    >'+title+ '</a>';
      console.log("@@@@@@@@111"+linkStr);
      insertContentHtml(linkStr,true);
}

RE.insertLinkNoTitle =function(linkUrl,imgUrl,isCustomTitle){
    var linkStr =
     '<p><br></p>'+ '<div class="w-e_link-card" card-des="'+linkUrl+'" data-module-name="link-card" has-delete="true" contenteditable="false" >'
      +'<div class="w-e_link-card_link-wrap" >'
      +'<img class="w-e_link-card_img" src="'+imgUrl+'" >'
      +'<div class="w-e_link-card_detail" >'
      +'<span class="w-e_link-card_address w-e_hide-font--nowrap">'+linkUrl+'</span>'
      + '</div>'
      + '</div>'
      +'<img id="img-delete" src="baseImg/post_delete.png" style="visibility: visible;" ontouchend="removeImg(this.parentNode)"/>'
      + '</div>'
      +'<p><br></p>';
      console.log("@@@@@@@@222"+linkStr);
      insertContentHtml(linkStr,true);
}

RE.insertLinkPost =function(postId,title,imgUrl,isCustomTitle,headUrl,userName){
    var linkStr =
     '<p><br></p>'+ '<div class="w-e_link-card" card-des="'+postId+'" data-module-name="link-card" has-delete="true" contenteditable="false" >'
      +'<div class="w-e_link-card_link-wrap" >'
      +'<img class="w-e_link-card_img" src="'+imgUrl+'" >'
      +'<div class="w-e_link-card_detail" >'
      +'<span class="w-e_link-card_title w-e_hide-font--nowrap" custom-title="'+isCustomTitle+'">'+title+'</span>'
      + '</div>'
      + '</div>'
      +'<div class="w-e_link-card_content">'
      +'<img class="w-e_link-card_content_img " src="'+headUrl+'">'
      +'<span class="w-e_link-card_content_text w-e_hide-font--nowrap">'+userName+'</span>'
      + '</div>'
      +'<img id="img-delete" src="baseImg/post_delete.png" style="visibility: visible;" ontouchend="removeImg(this.parentNode)"/>'
      + '</div>'
      +'<p><br></p>';
      console.log("@@@@@@@@333"+linkStr);
      insertContentHtml(linkStr,true);
}

RE.insertVideoLinkPost =function(postId,title,imgUrl,isCustomTitle,headUrl,userName){
//    var linkStr =
//      '<a class="k-common-link" href="#" data-kr-link="'+headUrl+'"  contenteditable="false">'
//      +title
//      + '</a>';
//      console.log("@@@@@@@@333"+linkStr);
//      insertContentHtml(linkStr,true);
      // 获取当前光标位置的 Selection 对象
          var selection = window.getSelection();
          if(selection.rangeCount>0){
              var range = selection.getRangeAt(0);

              var container = document.createElement('a');
              container.classList.add("k-common-link");
              container.setAttribute("href", "#");
              container.setAttribute("data-kr-link", headUrl);
              container.contentEditable = 'false';
              var text = document.createTextNode(title);
              container.appendChild(text);

              var rangeContainer = range.commonAncestorContainer;
              console.log("TxtContent : "+rangeContainer.innerHTML)
              console.log("TxtContent 2222: "+rangeContainer.nodeName )
              if(rangeContainer &&rangeContainer.innerHTML && rangeContainer.innerHTML!=''){
                if(rangeContainer.nodeName == 'DIV'){
                    [...rangeContainer.children].forEach(child => {
                        console.log("TxtContent child: "+child.innerHTML+"   "+child.nodeName)
                        if (
                          child.nodeName === 'P' &&
                          (child.innerHTML === '<br>' || child.innerHTML === '')
                        ) {
                          child.innerHTML = ''
                          child.appendChild(container)
                        }
                    });
                }else if(
                    rangeContainer.nodeName === 'P' &&
                    (rangeContainer.innerHTML === '<br>' || rangeContainer.innerHTML === '')
                ){
                    rangeContainer.innerHTML = ''
                    rangeContainer.appendChild(container)
                }
              }
              if (container.parentNode == null) {
                 range.insertNode(container);
              }

               var newRange = document.createRange();
               newRange.setStartAfter(container);
               newRange.collapse(true);

               var selection = window.getSelection();
               selection.removeAllRanges();
               selection.addRange(newRange);
              RE.handleNum();
              RE.callback();
          }
}

function clickAt(link){
//    var data=link.dataset.des
    window.heroweb.atClick(link);
    console.log("@@@@@@@@->"+link);
//    link.onclick = function(){
//        if(data.indexOf('http')!=-1){
//            window.heroweb.cardNetClick(data);
//        }else if(data.indexOf('000000')!=-1){
//            var topicid=des.split('000000')
//            window.heroweb.cardTopicClick(topicid[1]);
//        }else{
//            window.heroweb.cardPostClick(data);
//        }
//    }
}

function clickALink(link){
//    var data=link.dataset.des
    window.heroweb.cardTopicClick(link);
    console.log("@@@@@@@@->"+link);
//    link.onclick = function(){
//        if(data.indexOf('http')!=-1){
//            window.heroweb.cardNetClick(data);
//        }else if(data.indexOf('000000')!=-1){
//            var topicid=des.split('000000')
//            window.heroweb.cardTopicClick(topicid[1]);
//        }else{
//            window.heroweb.cardPostClick(data);
//        }
//    }
}

function clickNetImg(img){
    img.onclick = function(){
    var imgUrl=img.src.split('?');
	window.heroweb.imgClick(imgUrl[0]);
   }
}

function clickPostCard(div){
    div.onclick = function(){
	window.heroweb.cardPostClick(div.dataset.postid);
   }
}

function clickNetCard(div){
    div.onclick = function(){
	window.heroweb.cardNetClick(div.dataset.link);
   }
}

function clickTopicCard(div){
    div.onclick = function(){
	window.heroweb.cardTopicClick(div.dataset.topicid);
   }
}

function clickVideoLinkCard(div){
    div.onclick = function(){
	window.heroweb.cardNetClick(div.getAttribute("data-kr-link"));
   }
}


function clickGifImg(img){
    img.onclick = function(){
        var gifUrl=img.src.split('?');
    	window.heroweb.imgClick(gifUrl[0]);
   }
}

function removeImg(img){
    img.onclick = function(){
         var cover = img.querySelector("#img-cover-select");
           if(cover!=null){
    	        window.heroweb.onImgDelete(cover.alt);
            }
    RE.blurFocus();
    img.remove();
    txtChangeCallBack();
   }
}

function editCover(img){
    var imgUrl=img.alt;
    	window.heroweb.editCover(imgUrl);
}

function cancelCover(img){
        var imgUrl=img.alt;
    	window.heroweb.cancelCover(imgUrl);
}

function reCover(img){
    var imgUrl=img.alt;
    	window.heroweb.reCover(imgUrl);
}

const WETextViewTools = {
  /* 图片加载状态管理 */
  imageDoLoad(e) {
    const imgDom = e.target
    imgDom.removeEventListener('click', WETextViewTools.imageDoLoad)
    imgDom.classList.add('w_e_network_image')
    const src = imgDom.src
    imgDom.removeAttribute('src')
    imgDom.src = src
  },
  imageLoadSuccess(imgDom) {
    imgDom.removeEventListener('click', WETextViewTools.imageDoLoad)
    imgDom.classList.remove('w_e_img_load--error')
    imgDom.classList.remove('w_e_network_image')
    imgDom.classList.add('w_e_network_image_success')
  },
  imageLoadError(imgDom) {
    imgDom.addEventListener('click', WETextViewTools.imageDoLoad)
    imgDom.classList.remove('w_e_network_image')
    imgDom.classList.add('w_e_img_load--error')
  },
}

window.WETextViewTools = WETextViewTools
window.WEImageLoadError = window.WEImageLoadError || WETextViewTools.imageLoadError
window.WEImageLoadSuccess = window.WEImageLoadSuccess || WETextViewTools.imageLoadSuccess

//自动滚动处理
RE.getCaretYPosition = function() {
    var sel = window.getSelection();
    // Next line is comented to prevent deselecting selection. It looks like work but if there are any issues will appear then uconmment it as well as code above.
    //sel.collapseToStart();
    var range = sel.getRangeAt(0);
    var span = document.createElement('span');// something happening here preventing selection of elements
    range.collapse(false);
    range.insertNode(span);
    var topPosition = span.offsetTop;
    var spanParent = span.parentNode;
    spanParent.removeChild(span);
    spanParent.normalize();
//    span.parentNode.removeChild(span);
    return topPosition;
}

/**
 * 动画垂直滚动到页面指定位置
 * @param { Number } currentY 当前位置
 * @param { Number } targetY 目标位置
 */
function scrollAnimation(currentY, targetY) {
  // 获取当前位置方法
  // const currentY = document.documentElement.scrollTop || document.body.scrollTop

  // 计算需要移动的距离
  let needScrollTop = targetY - currentY
  let _currentY = currentY
  setTimeout(() => {
    // 一次调用滑动帧数，每次调用会不一样
    const dist = Math.ceil(needScrollTop / 10)
    _currentY += dist
    window.scrollTo(_currentY, currentY)
    // 如果移动幅度小于十个像素，直接移动，否则递归调用，实现动画效果
    if (needScrollTop > 10 || needScrollTop < -10) {
      scrollAnimation(_currentY, targetY)
    } else {
      window.scrollTo(_currentY, targetY)
    }
  }, 1)
}

RE.autoScroll = function(isImg){
    var a = restoreStyle();
console.log("######"+(typeof isImg));
    setTimeout(function() {
       const topY = RE.getCaretYPosition();
        console.log("getCaretYPosition光标"+topY);
           const currentY = document.documentElement.scrollTop || document.body.scrollTop
           const allTop=document.body.scrollHeight;
            console.log("allTop"+allTop);
           if(isImg){
              scrollAnimation(currentY, topY);
              console.log("g*******");
           }else if(allTop-topY<50){
               scrollAnimation(currentY, topY);
               console.log("g2*******");
           }
           console.log("currentY"+currentY);
           followState(a);
           console.log("****"+a);
   },800)
}
RE.emojiAutoScroll = function(){
       const topY = RE.getCaretYPosition();
       console.log("getCaretYPosition光标"+topY);
       const currentY = document.documentElement.scrollTop || document.body.scrollTop
       const allTop=document.body.scrollHeight;
       scrollAnimation(currentY, topY);
}

RE.scrollBottom = function(){
    window.scrollTo(0, document.body.scrollHeight);
}

RE.ListenerPaste = document.querySelector('div[contenteditable="true"]').addEventListener("paste", function(e) {
        e.stopPropagation();
        e.preventDefault();
        var text = '', event = (e.originalEvent || e);
        if (event.clipboardData && event.clipboardData.getData) {
            text = event.clipboardData.getData('text/plain');
        } else if (window.clipboardData && window.clipboardData.getData) {
            text = window.clipboardData.getData('Text');
        }
        console.log("+=====粘贴");
        text = text.replace(/<(style|script|iframe)[^>]*?>[\s\S]+?<\/\1\s*>/gi,'').replace(/<[^>]+?>/g,'').replace(/\s+/g,' ').replace(/ /g,' ').replace(/>/g,' ');        if (document.queryCommandSupported('insertText')) {
            document.execCommand('insertText', false, text);
        } else {
            document.execCommand('paste', false, text);
        }
});



